package jpasswordrecoverytool;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import javax.swing.*;

public class BruterUI extends JFrame {

    private static final int width = 800, height = 300;
    //flags
    private static boolean isRunning = false;
    private boolean isShown = false;
    private boolean isPaused = false;
    //menu and tray components
    private SystemTray tray;
    private TrayIcon trayIcon;
    private JMenuBar menuBar;
    private JMenu menuFile, menuHelp;
    private static JMenuItem menuItemExit, menuItemAbout, menuItem7z, menuItemUpdate;
    Image icon7z = new ImageIcon("resources" + separtor + "7ziplogo.png").getImage().getScaledInstance(24, 24, Image.SCALE_SMOOTH);
    Image iconUpdate = new ImageIcon("resources" + separtor + "windows_update_icon.png").getImage().getScaledInstance(24, 24, Image.SCALE_SMOOTH);
    Image iconAbout = new ImageIcon("resources" + separtor + "About-icon.png").getImage().getScaledInstance(24, 24, Image.SCALE_SMOOTH);
    Image iconExit = new ImageIcon("resources" + separtor + "exit.png").getImage().getScaledInstance(24, 24, Image.SCALE_SMOOTH);
    //password recovery components
    private JPanel archiveRecoveryPanel, archiveNorthJPanel, archiveSouthJPanel, archiveSouthInnerLeftJPanel, archiveSouthInnerRightJPanel;
    private JTextArea archiveConsoleJTextArea;
    private JScrollPane archiveConsoleScrollPane;
    private JButton archiveStartRecoveryJButton, archivePauseRecoveryJButton, archiveStopRecoveryJButton, archiveChooserButton, sevenZChooserButton;
    private FileDialog programChooser, archiveChooser;
    private JTextField sevenZPathJTextField, archiveJTextField, passwordExcludeField;
    private JCheckBox passwordCheckBoxLowerCase, passwordCheckBoxUpperCase, passwordCheckBoxDigits, passwordCheckBoxSpecial;
    private JSpinner minPasswordLengthSpinner, maxPasswordLengthSpinner;
    private JLabel characterChoiceLabel, minPasswordLengthLabel, blankLabel, maxPasswordLengthLabel, passwordExcludeLabel;
    private String sevenzLocation, archiveLocation;
    private ArchiveBruter archiveBruter;
    private Thread archiveBruterThread;
    //hash recovery components
    private HashBruter hashBruter;
    private Thread hashBruterThread;
    private JPanel hashRecoveryPanel, hashSouthJPanel, hashSouthInnerRightJPanel, hashNorthJPanel, hashSouthInnerLeftJPanel;
    private JCheckBox hashCheckBoxDigits, hashCheckBoxLowerCase, hashCheckBoxSpecial, hashCheckBoxUpperCase;
    private JButton hashStartRecoveryJButton, hashPauseRecoveryJButton, hashStopRecoveryJButton;
    private JTextField hashJTextField;
    private JLabel minHashLengthLabel;
    private JSpinner minHashLengthSpinner;
    private JTextField hashExcludeField;
    private JLabel maxHashLengthLabel;
    private JScrollPane hashConsoleScrollPane;
    private JTextArea hashConsoleJTextArea;
    private JSpinner maxHashLengthSpinner;
    private JLabel hashExcludeLabel, hashLabel;
    private JTabbedPane tabbedPane;
    public static final String separtor = File.separator;
    public static final String NEW_LINE = "\n";
    private String hash;
    private String hashType = "md5";
    private JComboBox hashTypeComboBox;
    private JLabel hashComboboxLabel;

    public BruterUI() {
        createGUI();
    }

    private void createGUI() {
        setTitle(Main.APP_NAME + " by " + Main.AUTHOR);
        setIconImage(new ImageIcon("resources" + separtor + "icon.png").getImage());
        setSize(width, height);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        setLocationRelativeTo(null);
        setResizable(false);

        addComponentsToContentPane(this.getContentPane());

        addAnonymousActionListeners();

        pack();

    }

    private void addComponentsToContentPane(Container contentPane) {
        readyComponents();

        menuFile.add(menuItemExit);
        menuHelp.add(menuItem7z);
        menuHelp.add(menuItemUpdate);
        menuHelp.add(menuItemAbout);
        menuBar.add(menuFile);
        menuBar.add(menuHelp);

        if (System.getProperty("os.name").contains("Windows")) {
            addArchiveComponentsToPanel();
            tabbedPane.insertTab("Compressed Archive Password Recovery",
                    new ImageIcon("resources" + separtor + "compressed-archive.png"),
                    archiveRecoveryPanel, "Allows you to recover lost passwords from archives like: .rar, .zip, .7z etc.",
                    0);
            addHashComponentsToPanel();


            tabbedPane.insertTab("MD5 and SHA-1/2 Hash Recovery",
                    new ImageIcon("resources" + separtor + "hash-icon.png"),
                    hashRecoveryPanel, "Allows you to recover lost passwords from MD5 and SHA-1 and SHA-2 based hashes",
                    1);
        } else {
            addHashComponentsToPanel();


            tabbedPane.insertTab("MD5 and SHA-1/2 Hash Recovery",
                    new ImageIcon("resources" + separtor + "hash-icon.png"),
                    hashRecoveryPanel, "Allows you to recover lost passwords from MD5 and SHA-1 and SHA-2 based hashes",
                    0);
        }
        contentPane.add(menuBar, BorderLayout.NORTH);
        contentPane.add(tabbedPane);

    }

    private void readyComponents() {

        prepareMenuBarAndSysTrayComponents();

        prepareArhiveRecoveryComponents();

        prepareHashRecoveryComponents();

    }

    private void prepareMenuBarAndSysTrayComponents() {
        trayIcon = new TrayIcon(new ImageIcon("resources/icon.png").getImage().getScaledInstance(16, 16, Image.SCALE_DEFAULT), Main.APP_NAME + " by " + Main.AUTHOR);

        if (!SystemTray.isSupported()) {
            JOptionPane.showMessageDialog(null, "SystemTray is not supported", "System tray not supported!", JOptionPane.ERROR_MESSAGE);
        } else {
            tray = SystemTray.getSystemTray();
        }

        menuBar = new JMenuBar();
        menuFile = new JMenu("File");
        menuHelp = new JMenu("Help");

        menuItemAbout = new JMenuItem("About", new ImageIcon(iconAbout));
        menuItemExit = new JMenuItem("Exit", new ImageIcon(iconExit));
        menuItemUpdate = new JMenuItem("Check for updates", new ImageIcon(iconUpdate));
        menuItem7z = new JMenuItem("Get 7-Zip", new ImageIcon(icon7z));

        menuBar = new JMenuBar();
        menuFile = new JMenu("File");
        menuHelp = new JMenu("Help");

        menuItemAbout = new JMenuItem("About", new ImageIcon(iconAbout));
        menuItemExit = new JMenuItem("Exit", new ImageIcon(iconExit));
        menuItemUpdate = new JMenuItem("Check for updates", new ImageIcon(iconUpdate));
        menuItem7z = new JMenuItem("Get 7-Zip", new ImageIcon(icon7z));

        blankLabel = new JLabel();

        tabbedPane = new JTabbedPane();
    }

    private void prepareArhiveRecoveryComponents() {
        archiveRecoveryPanel = new JPanel(new BorderLayout(10, 10));

        archiveNorthJPanel = new JPanel(new GridLayout(2, 2));
        archiveSouthJPanel = new JPanel(new GridLayout(1, 2));

        archiveSouthInnerLeftJPanel = new JPanel();
        archiveSouthInnerRightJPanel = new JPanel(new GridLayout(8, 2));
        programChooser = new FileDialog(this, "7z.exe");
        archiveChooser = new FileDialog(this, "");

        passwordCheckBoxDigits = new JCheckBox("0-9");
        passwordCheckBoxLowerCase = new JCheckBox("a-z");
        passwordCheckBoxSpecial = new JCheckBox("~`!@#$%^&*()_-+=}]{[|\\\",/?>.<:;");
        passwordCheckBoxUpperCase = new JCheckBox("A-Z");

        archiveStartRecoveryJButton = new JButton("Start Password Recovery");
        archivePauseRecoveryJButton = new JButton("Pause Password Recovery");
        archiveStopRecoveryJButton = new JButton("Stop Password Recovery");
        archiveChooserButton = new JButton("Browse for archive (.7z, .zip, .rar)");
        sevenZChooserButton = new JButton("Browse for 7z.exe");

        sevenZPathJTextField = new JTextField("C:" + separtor + "Path" + separtor + "to" + separtor + "7-Zip" + separtor + "7z.exe");
        archiveJTextField = new JTextField("C:" + separtor + "Path" + separtor + "to" + separtor + "ziparchive" + separtor + "archive.*");
        sevenZPathJTextField.setEditable(false);
        archiveJTextField.setEditable(false);
        passwordExcludeField = new JTextField();

        passwordExcludeLabel = new JLabel("Exclude characters:");
        minPasswordLengthLabel = new JLabel("Minimum password length:");
        maxPasswordLengthLabel = new JLabel("Maximum password length:");
        characterChoiceLabel = new JLabel("Character sets:");

        minPasswordLengthSpinner = new JSpinner();
        maxPasswordLengthSpinner = new JSpinner();

        archiveConsoleJTextArea = new JTextArea(13, 30);
        archiveConsoleScrollPane = new JScrollPane(archiveConsoleJTextArea);

        passwordExcludeField.setToolTipText("The characters to be excluded from the current chosen character sets i.e. ae1Q*");
        passwordExcludeLabel.setToolTipText("The characters to be excluded from the current chosen character sets i.e ae1Q*");
        minPasswordLengthLabel.setToolTipText("Represents the length at which to start the password recovery");
        maxPasswordLengthLabel.setToolTipText("Represents the length at which to stop attempting the recovery");
        minPasswordLengthSpinner.setToolTipText("Represents the length at which to start the password recovery");
        maxPasswordLengthSpinner.setToolTipText("Represents the length at which to stop attempting the recovery");
        archiveStartRecoveryJButton.setToolTipText("This will start an instance and attempt to recover the password under the limitations");
        archivePauseRecoveryJButton.setToolTipText("This will pause a current instance as to allow less processor usage if needed");
        archiveStopRecoveryJButton.setToolTipText("This will immediately halt the current recovery instance");
        archiveChooserButton.setToolTipText("Allows you to browse for the compressed archive to recover its password");
        sevenZChooserButton.setToolTipText("Browse for the location of the 7z.exe which is located in 7zips install directory");
        sevenZPathJTextField.setToolTipText("Location of 7z.exe which is located in 7zip program directory");
        archiveJTextField.setToolTipText("Path of the archive whos' password must be recovered");

        archivePauseRecoveryJButton.setEnabled(false);
        archiveStopRecoveryJButton.setEnabled(false);

        archiveConsoleJTextArea.setEditable(false);
        archiveConsoleJTextArea.setWrapStyleWord(true);
    }

    private void addArchiveComponentsToPanel() {
        archiveNorthJPanel.add(sevenZPathJTextField);
        archiveNorthJPanel.add(sevenZChooserButton);
        archiveNorthJPanel.add(archiveJTextField);
        archiveNorthJPanel.add(archiveChooserButton);

        archiveSouthInnerLeftJPanel.add(archiveConsoleScrollPane);

        archiveSouthInnerRightJPanel.add(characterChoiceLabel);
        archiveSouthInnerRightJPanel.add(blankLabel);
        archiveSouthInnerRightJPanel.add(passwordCheckBoxLowerCase);
        archiveSouthInnerRightJPanel.add(passwordCheckBoxUpperCase);
        archiveSouthInnerRightJPanel.add(passwordCheckBoxDigits);
        archiveSouthInnerRightJPanel.add(passwordCheckBoxSpecial);
        archiveSouthInnerRightJPanel.add(passwordExcludeLabel);
        archiveSouthInnerRightJPanel.add(passwordExcludeField);
        archiveSouthInnerRightJPanel.add(minPasswordLengthLabel);
        archiveSouthInnerRightJPanel.add(minPasswordLengthSpinner);
        archiveSouthInnerRightJPanel.add(maxPasswordLengthLabel);
        archiveSouthInnerRightJPanel.add(maxPasswordLengthSpinner);
        archiveSouthInnerRightJPanel.add(archiveStartRecoveryJButton);
        archiveSouthInnerRightJPanel.add(archivePauseRecoveryJButton);
        archiveSouthInnerRightJPanel.add(archiveStopRecoveryJButton);

        archiveSouthJPanel.add(archiveSouthInnerLeftJPanel);
        archiveSouthJPanel.add(archiveSouthInnerRightJPanel);

        archiveRecoveryPanel.add(archiveNorthJPanel, BorderLayout.NORTH);
        archiveRecoveryPanel.add(archiveSouthJPanel, BorderLayout.SOUTH);
    }

    private void prepareHashRecoveryComponents() {
        hashRecoveryPanel = new JPanel(new BorderLayout(10, 10));

        hashNorthJPanel = new JPanel(new GridLayout(2, 1));
        hashSouthJPanel = new JPanel(new GridLayout(1, 2));

        hashSouthInnerLeftJPanel = new JPanel();
        hashSouthInnerRightJPanel = new JPanel(new GridLayout(9, 2));


        hashCheckBoxDigits = new JCheckBox("0-9");
        hashCheckBoxLowerCase = new JCheckBox("a-z");
        hashCheckBoxSpecial = new JCheckBox("~`!@#$%^&*()_-+=}]{[|\\\",/?>.<:;");
        hashCheckBoxUpperCase = new JCheckBox("A-Z");

        hashStartRecoveryJButton = new JButton("Start Hash Recovery");

        hashPauseRecoveryJButton = new JButton("Pause Hash Recovery");
        hashStopRecoveryJButton = new JButton("Stop Hash Recovery");

        hashJTextField = new JTextField("");
        hashExcludeField = new JTextField();

        hashExcludeLabel = new JLabel("Exclude characters:");
        minHashLengthLabel = new JLabel("Minimum hash length:");
        maxHashLengthLabel = new JLabel("Maximum hash length:");
        characterChoiceLabel = new JLabel("Character sets:");

        minHashLengthSpinner = new JSpinner();
        maxHashLengthSpinner = new JSpinner();

        hashLabel = new JLabel("Enter hash here");
        hashConsoleJTextArea = new JTextArea(13, 30);
        hashConsoleScrollPane = new JScrollPane(hashConsoleJTextArea);

        hashExcludeField.setToolTipText("The characters to be excluded from the current chosen character sets i.e. ae1Q*");
        hashExcludeLabel.setToolTipText("The characters to be excluded from the current chosen character sets i.e ae1Q*");
        minHashLengthLabel.setToolTipText("Represents the length at which to start the hash recovery");
        maxHashLengthLabel.setToolTipText("Represents the length at which to stop attempting the hash recovery");
        minHashLengthSpinner.setToolTipText("Represents the length at which to start the hash recovery");
        maxHashLengthSpinner.setToolTipText("Represents the length at which to stop attempting the hash recovery");
        hashStartRecoveryJButton.setToolTipText("This will start an instance and attempt to recover the hash under the limitations");
        hashPauseRecoveryJButton.setToolTipText("This will pause a current instance as to allow less processor usage if needed");
        hashStopRecoveryJButton.setToolTipText("This will immediately halt the current hash recovery instance");
        hashJTextField.setToolTipText("Hash of password to be bruteforced for collision");

        hashPauseRecoveryJButton.setEnabled(false);
        hashStopRecoveryJButton.setEnabled(false);

        hashConsoleJTextArea.setEditable(false);
        hashConsoleJTextArea.setWrapStyleWord(true);
        String s[] = {"MD2","MD5", "SHA-1", "SHA-256", "SHA-384", "SHA-512","CRC16","CRC32","CRC64","Adler32"};
        hashTypeComboBox = new JComboBox(s);
        hashComboboxLabel = new JLabel("Hash type");

    }

    private void addHashComponentsToPanel() {

        hashNorthJPanel.add(hashLabel);
        hashNorthJPanel.add(hashJTextField);

        hashSouthInnerLeftJPanel.add(hashConsoleScrollPane);

        hashSouthInnerRightJPanel.add(characterChoiceLabel);
        hashSouthInnerRightJPanel.add(blankLabel);
        hashSouthInnerRightJPanel.add(hashCheckBoxLowerCase);
        hashSouthInnerRightJPanel.add(hashCheckBoxUpperCase);
        hashSouthInnerRightJPanel.add(hashCheckBoxDigits);
        hashSouthInnerRightJPanel.add(hashCheckBoxSpecial);
        hashSouthInnerRightJPanel.add(hashComboboxLabel);
        hashSouthInnerRightJPanel.add(hashTypeComboBox);
        hashSouthInnerRightJPanel.add(hashExcludeLabel);
        hashSouthInnerRightJPanel.add(hashExcludeField);
        hashSouthInnerRightJPanel.add(minHashLengthLabel);
        hashSouthInnerRightJPanel.add(minHashLengthSpinner);
        hashSouthInnerRightJPanel.add(maxHashLengthLabel);
        hashSouthInnerRightJPanel.add(maxHashLengthSpinner);
        hashSouthInnerRightJPanel.add(hashStartRecoveryJButton);
        hashSouthInnerRightJPanel.add(hashPauseRecoveryJButton);
        hashSouthInnerRightJPanel.add(hashStopRecoveryJButton);

        hashSouthJPanel.add(hashSouthInnerLeftJPanel);
        hashSouthJPanel.add(hashSouthInnerRightJPanel);

        hashRecoveryPanel.add(hashNorthJPanel, BorderLayout.NORTH);
        hashRecoveryPanel.add(hashSouthJPanel, BorderLayout.SOUTH);
    }

    private void addAnonymousActionListeners() {

        this.addWindowListener(new WindowListener() {

            @Override
            public void windowOpened(WindowEvent e) {
            }

            @Override
            public void windowClosing(WindowEvent e) {
                int answer = JOptionPane.showConfirmDialog(null, "Are you sure you want to exit?", "Exit", JOptionPane.YES_NO_OPTION);
                if (JOptionPane.YES_OPTION == answer) {
                    if (isRunning) {
                        answer = JOptionPane.showConfirmDialog(null, "A recovery is in progress, would you like to quit anyway?", "Warning", JOptionPane.YES_NO_OPTION);
                        if (JOptionPane.YES_OPTION == answer) {
                            archiveStopRecoveryJButton.doClick();
                            hashStopRecoveryJButton.doClick();
                            dispose();
                        } else if (JOptionPane.NO_OPTION == answer) {
                        }
                    } else {
                        dispose();
                    }
                }
            }

            @Override
            public void windowClosed(WindowEvent e) {
            }

            @Override
            public void windowIconified(WindowEvent e) {
                int answer = JOptionPane.showConfirmDialog(null, "Minimize to system tray?", "Warning", JOptionPane.YES_NO_OPTION);
                if (JOptionPane.YES_OPTION == answer) {
                    try {
                        if (SystemTray.isSupported()) {
                            tray.add(trayIcon);
                            setVisible(false);
                        } else {
                            JOptionPane.showMessageDialog(null, "Your OS does not support Java SystemTray", "Error", JOptionPane.ERROR_MESSAGE);
                        }
                    } catch (AWTException ex) {
                        JOptionPane.showMessageDialog(null, "SystemTray is not supported", "System tray not supported!", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }

            @Override
            public void windowDeiconified(WindowEvent e) {
            }

            @Override
            public void windowActivated(WindowEvent e) {
            }

            @Override
            public void windowDeactivated(WindowEvent e) {
            }
        });

        trayIcon.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                tray.remove(trayIcon);
                setVisible(true);
            }
        });
        menuItemUpdate.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String url = "https://sourceforge.net/projects/jpassrecovery/files/JPasswordRecoveryTool%20" + Main.NEXT_APP_VERSION + ".7z/download/";
                if (Desktop.isDesktopSupported()) {
                    try {
                        Desktop.getDesktop().browse(new URI(url));
                    } catch (URISyntaxException | IOException ex) {
                        JOptionPane.showMessageDialog(null, "Error occured while attempting to open update url", "Error", JOptionPane.ERROR_MESSAGE);
                    }
                } else {
                    JOptionPane.showMessageDialog(null, "Desktop not supported, please go to: " + url + " to download the latest version of JPRT", "Warning", JOptionPane.WARNING_MESSAGE);
                }
            }
        });
        menuItem7z.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (Desktop.isDesktopSupported()) {
                    try {
                        Desktop.getDesktop().browse(new URI("http://www.7-zip.org/"));
                    } catch (URISyntaxException | IOException ex) {
                        JOptionPane.showMessageDialog(null, "Error occured while attempting to open www.7-zip.org url", "Error", JOptionPane.ERROR_MESSAGE);
                    }
                } else {
                    JOptionPane.showMessageDialog(null, "Desktop not supported, please go to: http://www.7-zip.org/ to download 7-Zip", "Warning", JOptionPane.WARNING_MESSAGE);
                }
            }
        });
        menuItemExit.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (isRunning) {
                    int answer = JOptionPane.showConfirmDialog(null, "A recovery is in progress, would you like to quit anyway?", "Quit while recovery in progress...", JOptionPane.YES_NO_OPTION);
                    if (answer == JOptionPane.YES_OPTION) {
                        archiveStopRecoveryJButton.doClick();
                        hashStopRecoveryJButton.doClick();
                        dispose();
                    }
                } else {
                    dispose();
                }
            }
        });
        menuItemAbout.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {

                    @Override
                    public void run() {
                        AboutBox aboutBox = new AboutBox();
                    }
                }).start();
            }
        });
        sevenZChooserButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                programChooser.setVisible(true);
                if (programChooser.getFile() != null && programChooser.getDirectory() != null) {
                    sevenzLocation = programChooser.getDirectory() + programChooser.getFile();
                    sevenZPathJTextField.setText(sevenzLocation);
                }
            }
        });
        archiveChooserButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                archiveChooser.setVisible(true);
                if (archiveChooser.getFile() != null && archiveChooser.getDirectory() != null) {
                    archiveLocation = archiveChooser.getDirectory() + archiveChooser.getFile();
                    archiveJTextField.setText(archiveLocation);

                }
            }
        });
        archiveStartRecoveryJButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                if (sevenzLocation == null || archiveLocation == null || archiveLocation.equals("") || sevenzLocation.equals("")) {
                    JOptionPane.showMessageDialog(null, "You did not choose the 7z.exe path and/or the zip archive is not chosen", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                if ((int) minPasswordLengthSpinner.getValue() <= 0 || (int) maxPasswordLengthSpinner.getValue() <= 0) {
                    JOptionPane.showMessageDialog(null, "Please select a min and max password length greater than 0", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (!passwordCheckBoxDigits.isSelected() && !passwordCheckBoxUpperCase.isSelected() && !passwordCheckBoxLowerCase.isSelected() && !passwordCheckBoxSpecial.isSelected()) {
                    JOptionPane.showMessageDialog(null, "Please select atleast 1 character set", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (isRunning) {
                    JOptionPane.showMessageDialog(null, "An instance is already running, please Stop it, then try again", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                Runnable r = createArchiveThread();
                if (r == null) {
                    JOptionPane.showMessageDialog(null, "A problem occured while starting the instance. Could be becuase to many characters were excluded!", "Error", JOptionPane.WARNING_MESSAGE);
                    return;
                }

                archiveBruterThread = new Thread(r);
                archiveBruterThread.start();
                Thread thread = new Thread(getStatus(archiveBruter));
                thread.start();
                isRunning = true;
                archivePauseRecoveryJButton.setEnabled(true);
                archiveStopRecoveryJButton.setEnabled(true);
            }
        });
        archivePauseRecoveryJButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (!isPaused) {
                    archivePauseRecoveryJButton.setText("Un-Pause Password Recovery");
                    isPaused = true;
                    archiveBruter.setPaused(true);
                    archiveStartRecoveryJButton.setEnabled(false);
                } else {
                    archivePauseRecoveryJButton.setText("Pause Password Recovery");
                    archiveBruter.setPaused(false);
                    isPaused = false;
                    archiveStartRecoveryJButton.setEnabled(true);
                }
            }
        });
        archiveStopRecoveryJButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (isPaused) {
                    archivePauseRecoveryJButton.doClick();
                }
                archivePauseRecoveryJButton.setEnabled(false);
                archiveStartRecoveryJButton.setEnabled(true);
                archiveBruter.setIsDone(true);
                isRunning = false;
                archiveBruter.setFound(true);
                clearPasswordText();
            }
        });
        hashStartRecoveryJButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                hash = hashJTextField.getText();
                if (hash.equals("")) {
                    JOptionPane.showMessageDialog(null, "You did not enter a correct hash", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                if ((int) minHashLengthSpinner.getValue() <= 0 || (int) maxHashLengthSpinner.getValue() <= 0) {
                    JOptionPane.showMessageDialog(null, "Please select a min and max hash length greater than 0", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (!hashCheckBoxDigits.isSelected() && !hashCheckBoxUpperCase.isSelected() && !hashCheckBoxLowerCase.isSelected() && !hashCheckBoxSpecial.isSelected()) {
                    JOptionPane.showMessageDialog(null, "Please select atleast 1 character set", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (isRunning) {
                    JOptionPane.showMessageDialog(null, "An instance is already running, please Stop it, then try again", "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }

                Runnable r = createHashThread();
                if (r == null) {
                    JOptionPane.showMessageDialog(null, "A problem occured while starting the instance. Could be becuase to many characters were excluded!", "Error", JOptionPane.WARNING_MESSAGE);
                    return;
                }

                hashBruterThread = new Thread(r);
                hashBruterThread.start();
                Thread thread = new Thread(getStatus(hashBruter));
                thread.start();
                isRunning = true;
                hashPauseRecoveryJButton.setEnabled(true);
                hashStopRecoveryJButton.setEnabled(true);
            }
        });
        hashPauseRecoveryJButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (!isPaused) {
                    hashPauseRecoveryJButton.setText("Un-Pause Hash Recovery");
                    isPaused = true;
                    hashBruter.setPaused(true);
                    hashStartRecoveryJButton.setEnabled(false);
                } else {
                    hashPauseRecoveryJButton.setText("Pause Hash Recovery");
                    hashBruter.setPaused(false);
                    isPaused = false;
                    hashStartRecoveryJButton.setEnabled(true);
                }
            }
        });
        hashStopRecoveryJButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                if (isPaused) {
                    hashPauseRecoveryJButton.doClick();
                }
                hashPauseRecoveryJButton.setEnabled(false);
                hashStopRecoveryJButton.setEnabled(false);
                hashStartRecoveryJButton.setEnabled(true);
                hashBruter.setIsDone(true);
                isRunning = false;
                hashBruter.setFound(true);
                clearHashText();
            }
        });
        hashTypeComboBox.addItemListener(new ItemListener() {

            int count = 0;

            @Override
            public void itemStateChanged(ItemEvent ie) {
                if (count == 0) {
                    count++;
                } else if (count == 1) {
                    hashType = (String) hashTypeComboBox.getSelectedItem();
                    count = 0;
                }
            }
        });
    }

    private Runnable getStatus(Object o) {
        Runnable r = null;
        if (o instanceof ArchiveBruter) {
            r = new Runnable() {

                @Override
                public void run() {
                    while (!archiveBruter.isFound() & !archiveBruter.isDone()) {
                        if (!isPaused) {
                            clearPasswordText();
                            setPasswordText("Password: " + archiveBruter.getGeneratedPassword());
                            if( archiveBruter.getGeneratedPassword() != null) {
                            setPasswordText("Password length: " + archiveBruter.getGeneratedPassword().length());
                            }
                            setPasswordText("Number of Possibilities: " + archiveBruter.getNumberOfPossibilities());
                            setPasswordText("Estimated passwords left: " + archiveBruter.getRemainder());
                            archiveBruter.setEndtime(System.nanoTime());
                            setPasswordText("Checked passwords: " + archiveBruter.getCounter() + " @" + archiveBruter.getPerSecond() + "p.p.s");
                            setPasswordText("Time elapsed: " + archiveBruter.calculateTimeElapsed());
                            try {
                                Thread.sleep(500);
                            } catch (InterruptedException ex) {
                                ex.printStackTrace();
                            }

                        }
                        if (archiveBruter.isFound() && archiveBruter.isDone()) {
                            isRunning = false;
                            archiveStopRecoveryJButton.setEnabled(false);
                            archivePauseRecoveryJButton.setEnabled(false);
                            clearPasswordText();
                            setPasswordText("Number of Possibilities: " + archiveBruter.getNumberOfPossibilities());
                            setPasswordText("Password length: " + archiveBruter.getGeneratedPassword().length());
                            setPasswordText("Checked passwords: " + archiveBruter.getCounter());
                            archiveBruter.setEndtime(System.nanoTime());
                            setPasswordText("Time elapsed: " + archiveBruter.calculateTimeElapsed());
                            setPasswordText("Found password: " + archiveBruter.getPassword());
                        } else if (archiveBruter.isDone()) {
                            isRunning = false;
                            archiveStopRecoveryJButton.setEnabled(false);
                            archivePauseRecoveryJButton.setEnabled(false);
                            clearPasswordText();
                            setPasswordText("Number of Possibilities: " + archiveBruter.getNumberOfPossibilities());
                            setPasswordText("Password length: " + archiveBruter.getGeneratedPassword().length());
                            setPasswordText("Checked passwords: " + archiveBruter.getCounter());
                            archiveBruter.setEndtime(System.nanoTime());
                            setPasswordText("Time elapsed: " + archiveBruter.calculateTimeElapsed());
                            setPasswordText("Could not find any passwords between " + archiveBruter.getMinLength() + "-" + archiveBruter.getMaxLength());
                        }
                    }


                }
            };
        } else if (o instanceof HashBruter) {
            r = new Runnable() {

                @Override
                public void run() {

                    while (!hashBruter.isFound() & !hashBruter.isDone()) {
                        if (!isPaused) {
                            clearHashText();
                            setHashText("Hash: " + hashBruter.getGeneratedHash());
                            setHashText("Number of Possibilities: " + hashBruter.getNumberOfPossibilities());
                            if(hashBruter.getGeneratedHash() != null) {
                            setHashText("Charcters length: " + hashBruter.getPassword().length());
                            }
                            hashBruter.setEndtime(System.nanoTime());
                            setHashText("Checked hashes: " + hashBruter.getCounter() + " @" + hashBruter.getPerSecond() + "h.p.s");
                            setHashText("Time elapsed: " + hashBruter.calculateTimeElapsed());
                            setHashText("Estimated hashes left: " + hashBruter.getRemainder());
                            try {
                                Thread.sleep(500);
                            } catch (InterruptedException ex) {
                                ex.printStackTrace();
                            }
                        }
                    }

                    if (hashBruter.isFound() && hashBruter.isDone()) {
                        hashStopRecoveryJButton.setEnabled(false);
                        hashPauseRecoveryJButton.setEnabled(false);
                        isRunning = false;
                        clearHashText();
                        setHashText("Number of Possibilities: " + hashBruter.getNumberOfPossibilities());
                            setHashText("Charcters length: " + hashBruter.getPassword().length());
                        setHashText("Checked hashes: " + hashBruter.getCounter());
                        hashBruter.setEndtime(System.nanoTime());
                        setHashText("Time elapsed: " + hashBruter.calculateTimeElapsed());
                        setHashText("Found collision: " + hashBruter.getGeneratedHash() + " password is: " + hashBruter.getPassword());
                    } else if (hashBruter.isDone()) {
                        isRunning = false;
                        hashStopRecoveryJButton.setEnabled(false);
                        hashPauseRecoveryJButton.setEnabled(false);
                        clearHashText();
                        setHashText("Number of Possibilities: " + hashBruter.getNumberOfPossibilities());
                            setHashText("Charcters  length: " + hashBruter.getPassword().length());
                        setHashText("Checked Hashes: " + hashBruter.getCounter());
                        hashBruter.setEndtime(System.nanoTime());
                        setHashText("Time elapsed: " + hashBruter.calculateTimeElapsed());
                        setHashText("Could not find any collisions between " + hashBruter.getMinLength() + "-" + hashBruter.getMaxLength());
                    }
                }
            };
        }
        return r;
    }

    private void checkArchiveCheckBoxes(Object o) {
        if (passwordCheckBoxDigits.isSelected()) {
            archiveBruter.addDigits();
        }
        if (passwordCheckBoxLowerCase.isSelected()) {

            archiveBruter.addLowerCaseLetters();
        }
        if (passwordCheckBoxUpperCase.isSelected()) {

            archiveBruter.addUpperCaseLetters();
        }
        if (passwordCheckBoxSpecial.isSelected()) {

            archiveBruter.addSpecialCharacters();
        }
    }

    private void checkHashCheckBoxes(Object o) {
        if (hashCheckBoxDigits.isSelected()) {
            hashBruter.addDigits();
        }
        if (hashCheckBoxLowerCase.isSelected()) {

            hashBruter.addLowerCaseLetters();
        }
        if (hashCheckBoxUpperCase.isSelected()) {

            hashBruter.addUpperCaseLetters();
        }
        if (hashCheckBoxSpecial.isSelected()) {

            hashBruter.addSpecialCharacters();
        }
    }

    private Runnable createArchiveThread() {
        Runnable r = null;
        archiveBruter = new ArchiveBruter();
        archiveBruter.set7zLocation(sevenzLocation);
        archiveBruter.setArchiveLocation(archiveLocation);
        archiveBruter.setMinLength((int) minPasswordLengthSpinner.getValue());
        archiveBruter.setMaxLength((int) maxPasswordLengthSpinner.getValue());
        checkArchiveCheckBoxes(archiveBruter);
        if (archiveBruter.excludeChars(passwordExcludeField.getText())) {
            r = new Runnable() {

                @Override
                public void run() {
                    archiveBruter.tryBruteForce();
                }
            };
        }
        return r;
    }

    private Runnable createHashThread() {
        Runnable r = null;
        hashBruter = new HashBruter();
        hashBruter.setHash(hash);
        hashBruter.setType(hashType);
        checkHashCheckBoxes(hashBruter);
        hashBruter.setMinLength((int) minHashLengthSpinner.getValue());
        hashBruter.setMaxLength((int) maxHashLengthSpinner.getValue());
        if (hashBruter.excludeChars(hashExcludeField.getText())) {
            r = new Runnable() {

                @Override
                public void run() {
                    hashBruter.tryBruteForce();
                }
            };
        }
        return r;
    }

    @Override
    public void setVisible(boolean b) {
        super.setVisible(b);
        if (System.getProperty("os.name").contains("Windows")) {
            if (isShown == false) {
                autodetect7z();
                isShown = true;
            }
        }
    }

    private void autodetect7z() {
        String drive = System.getenv("homedrive") + separtor;
        boolean autoDetected = false;
        if (new File(drive + "Program Files (x86)" + separtor + "7-Zip" + separtor + "7z.exe").exists()) {
            autoDetected = true;
            sevenzLocation = drive + "Program Files (x86)" + separtor + "7-Zip" + separtor + "7z.exe";
            sevenZPathJTextField.setText(sevenzLocation);
        } else if (new File(drive + "7-Zip" + separtor + "7z.exe").exists()) {
            autoDetected = true;
            sevenzLocation = drive + "7-Zip" + separtor + "7z.exe";
            sevenZPathJTextField.setText(sevenzLocation);
        } else if (new File(drive + "Program Files" + separtor + "7-Zip" + separtor + "7z.exe").exists()) {
            autoDetected = true;
            sevenzLocation = drive + "Program Files" + separtor + "7-Zip" + separtor + "7z.exe";
            sevenZPathJTextField.setText(sevenzLocation);
        }
        if (autoDetected) {
            sevenZPathJTextField.setEnabled(false);
            sevenZChooserButton.setEnabled(false);
            //JOptionPane.showMessageDialog(null, "7z.exe location automatically detected!", "Auto detected 7zip", JOptionPane.INFORMATION_MESSAGE);
        } else {
            sevenZPathJTextField.setEnabled(false);
            JOptionPane.showMessageDialog(null, "7z.exe could not be found!\nTo get 7-zip goto Help->Get 7-Zip", "Auto detect 7zip FAILED", JOptionPane.ERROR_MESSAGE);
        }
    }

    void setPasswordText(String string) {

        String tmp = archiveConsoleJTextArea.getText();
        archiveConsoleJTextArea.setText(tmp + NEW_LINE + string);
    }

    void clearPasswordText() {
        archiveConsoleJTextArea.setText("");
    }

    void setHashText(String string) {

        String tmp = hashConsoleJTextArea.getText();
        hashConsoleJTextArea.setText(tmp + NEW_LINE + string);
    }

    void clearHashText() {
        hashConsoleJTextArea.setText("");
    }
}
